@@ -5,21 +5,21 @@ from __future__ import annotations
 
 import requests
 import logging
-from fastapi import FastAPI, Request
+from fastapi import FastAPI, HTTPException, Request
 from uvicorn import run as _run
 from .constants import VERSION
-from . import ext
 from .ext._property import authorized
 from .ext._send_others import send_custom_json, send_contacts
 from .ext._message import send_template
 from .ext._send_media import send_image, send_video, send_audio, send_location, send_sticker, send_document
 from .ext._media import upload_media, query_media_url, download_media, delete_media
 from .ext._buttons import send_button, create_button, send_reply_button
-from .ext._static import is_message, get_mobile, get_author, get_name, get_message, get_message_id, get_message_type, get_message_timestamp, get_audio, get_delivery, get_document, get_image, get_interactive_response, get_location, get_video, changed_field
+from .ext._static import is_message, get_mobile, get_author, get_name, get_message, get_message_id, get_message_type, get_message_timestamp, get_audio, get_delivery, get_document, get_image, get_sticker, get_interactive_response, get_location, get_video, changed_field
+import json
 
 
 class WhatsApp(object):
-    def __init__(self, token: str = "", phone_number_id: str = "", logger: bool = True, update_check: bool = True):
+    def __init__(self, token: str = "", phone_number_id: str = "", logger: bool = True, update_check: bool = True, verify_token: str = "", debug: bool = True):
         """
         Initialize the WhatsApp Object
 
@@ -50,29 +50,21 @@ class WhatsApp(object):
                     else:
                         logging.critical(
                             f"Whatsapp-python is out of date. Please update to the latest version {latest}. READ THE CHANGELOG BEFORE UPDATING. NEW VERSIONS MAY BREAK YOUR CODE IF NOT PROPERLY UPDATED.")
-                if version_int > latest_int:
-                    latest_beta = int(str(requests.get(
-                        "https://raw.githubusercontent.com/filipporomani/whatsapp/main/.version").text).replace(".", ""))
-                    if latest_beta > version_int:
-                        logging.critical(
-                            "A new beta version is available. Please update to the latest version. READ THE CHANGELOG BEFORE UPDATING. NEW VERSIONS MAY BREAK YOUR CODE IF NOT PROPERLY UPDATED.")
-                    logging.critical(
-                        f"You are using a development version of whatsapp-python. Please report any issue on GitHub.")
 
-        
         if token == "":
             logging.error("Token not provided")
             raise ValueError("Token not provided but required")
         if phone_number_id == "":
-            logging.error("Phone number id not provided")
+            logging.error("Phone number ID not provided")
             raise ValueError(
                 "Phone number ID not provided but required")
         self.token = token
         self.phone_number_id = phone_number_id
         self.base_url = "https://graph.facebook.com/v18.0"
         self.url = f"{self.base_url}/{phone_number_id}/messages"
+        self.verify_token = verify_token
 
-        async def base():
+        async def base(*args):
             pass
         self.message_handler = base
         self.other_handler = base
@@ -84,6 +76,9 @@ class WhatsApp(object):
         if logger is False:
             logging.disable(logging.INFO)
             logging.disable(logging.ERROR)
+        if debug is False:
+            logging.disable(logging.DEBUG)
+            logging.disable(logging.ERROR)
 
         self.app = FastAPI()
 
@@ -91,32 +86,42 @@ class WhatsApp(object):
 
         @self.app.get("/")
         async def verify_endpoint(r: Request):
-            if r.query_params.get("hub.verify_token") == self.token:
-                logging.info("Verified webhook")
+            if r.query_params.get("hub.verify_token") == self.verify_token:
+                logging.debug("Webhook verified successfully")
                 challenge = r.query_params.get("hub.challenge")
                 self.verification_handler(challenge)
                 self.other_handler(challenge)
-                return str(challenge)
-            logging.error("Webhook Verification failed")
+                return int(challenge)
+            logging.error("Webhook Verification failed - token mismatch")
             await self.verification_handler(False)
             await self.other_handler(False)
             return {"success": False}
 
         @self.app.post("/")
         async def hook(r: Request):
-            # Handle Webhook Subscriptions
-            data = await r.json()
-            if data is None:
-                return {"success": False}
-            logging.info("Received webhook data: %s", data)
-            changed_field = self.instance.changed_field(data)
-            if changed_field == "messages":
-                new_message = self.instance.is_message(data)
-                if new_message:
-                    msg = Message(instance=self.instance, data=data)
-                    await self.message_handler(msg)
-                    await self.other_handler(msg)
-            return {"success": True}
+            try:
+                # Handle Webhook Subscriptions
+                data = await r.json()
+                if data is None:
+                    return {"success": False}
+                data_str = json.dumps(data, indent=4)
+                # log the data received only if the log level is debug
+                logging.debug(f"Received webhook data: {data_str}")
+
+                changed_field = self.changed_field(data)
+                if changed_field == "messages":
+                    new_message = self.is_message(data)
+                    if new_message:
+                        msg = Message(instance=self, data=data)
+                        await self.message_handler(msg)
+                        await self.other_handler(msg)
+                return {"success": True}
+            except Exception as e:
+                logging.error(f"Error parsing message: {e}")
+                raise HTTPException(status_code=500, detail={
+                    "success": False,
+                    "error": str(e)
+                })
 
     # all the files starting with _ are imported here, and should not be imported directly.
 
@@ -131,12 +136,30 @@ class WhatsApp(object):
     get_delivery = staticmethod(get_delivery)
     get_document = staticmethod(get_document)
     get_image = staticmethod(get_image)
+    get_sticker = staticmethod(get_sticker)
     get_interactive_response = staticmethod(get_interactive_response)
     get_location = staticmethod(get_location)
     get_video = staticmethod(get_video)
     changed_field = staticmethod(changed_field)
     get_author = staticmethod(get_author)
 
+    send_button = send_button
+    create_button = create_button
+    send_reply_button = send_reply_button
+    send_image = send_image
+    send_video = send_video
+    send_audio = send_audio
+    send_location = send_location
+    send_sticker = send_sticker
+    send_document = send_document
+    upload_media = upload_media
+    query_media_url = query_media_url
+    download_media = download_media
+    delete_media = delete_media
+    send_template = send_template
+    send_custom_json = send_custom_json
+    send_contacts = send_contacts
+
     authorized = property(authorized)
 
     def create_message(self, **kwargs) -> Message:
@@ -184,6 +207,10 @@ class WhatsApp(object):
 
 class Message(object):
     def __init__(self, id: int = None, data: dict = {}, instance: WhatsApp = None, content: str = "", to: str = "", rec_type: str = "individual"):  # type: ignore
+        self.instance = instance
+        self.url = self.instance.url
+        self.headers = self.instance.headers
+
         try:
             self.id = instance.get_message_id(data)
         except:
@@ -214,6 +241,11 @@ class Message(object):
                 self.image = self.instance.get_image(data)
             except:
                 self.image = None
+        if self.type == "sticker":
+            try:
+                self.sticker = self.instance.get_sticker(data)
+            except:
+                self.sticker = None
         elif self.type == "video":
             try:
                 self.video = self.instance.get_video(data)
@@ -240,8 +272,4 @@ class Message(object):
             except:
                 pass
 
-        self.instance = instance
-        self.url = self.instance.url
-        self.headers = self.instance.headers
-
-    from .ext._message import send, reply, mark_as_read
+    from .ext._message import send, reply, mark_as_read, react
